//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: DESymbolVisList.C
//  PROJ: COKO DEBUGGER
//  DATE: WINTER 2002                                             
//
// SYNOPSIS
//  Classes for Symbol Visual DIsplay List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _DSYMBOLVISLIST_C_
#define _DSYMBOLVISLIST_C_

#include "DESymbolVisList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DSymbolVisNode
// Parameters:	(char*, DWhichList*, int, DScrollWidget*, DPaintWindow*, int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DSymbolVisNode::DSymbolVisNode(const char * a, DWhichList* wl, int num, DScrollWidget* gb, DPaintWindow* pw, int index):	
  _next(NULL),
  _gb(NULL),
  _pw(NULL),
  _wl(NULL),
  _value(NULL)
{
  _num = num;
  if (a) {
    _value = new char[strlen(a) + 1];
    strcpy(_value, a);
  }
  else _value = NULL;
  _gb = gb;
  _pw = pw;
  _viewIndex = index;
  _wl = wl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~DSymbolVisNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
DSymbolVisNode::~DSymbolVisNode(void) 
{
  if (_value) { delete _value; _value = NULL; }
  _next = NULL;
  if (_pw) delete _pw;
  if (_gb) delete _gb;
  if (_wl) delete _wl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void DSymbolVisNode::display(void) 
{
  cout << _num << ": ";
  if (_value)
    cout <<  _value  << endl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (char*, DWhichList*, int, DScrollWidget*, DPaintWindow*, int)    
//
// Insert a new node with _value = a and _num = num to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::insert(const char* a, DWhichList* wl, int num, DScrollWidget* gb, DPaintWindow* pw, int index) 
{
  if (!_head)
	_head = _tail = new DSymbolVisNode(a, wl, num, gb, pw, index); 
  else 
	{
	  DSymbolVisNode * tmp = new DSymbolVisNode(a, wl, num, gb, pw, index);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::pop(void)
{
  if (!_head)
	return 0;

  DSymbolVisNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  delete tmpnode; tmpnode = NULL;
  
  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (void)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* DSymbolVisList::top(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ DSymbolVisList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* DSymbolVisList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextValue
// Parameters:	(void)
//
// Returns _value of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* DSymbolVisList::iteratorNextValue(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNum
// Parameters:	(void)
//
// Returns _num of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::iteratorNum(void)
{
  if (_iterator)
	return _iterator->_num;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextNum
// Parameters:	(void)
//
// Returns _num of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::iteratorNextNum(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_num;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setIteratorNum
// Parameters:	(int)
//
// Set _num of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::setIteratorNum(int x)
{
  if (_iterator)
	_iterator->_num = x;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setIteratorNextNum
// Parameters:	(int)
//
// Set _num of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::setIteratorNextNum(int x)
{
  if ((_iterator) && (_iterator->_next))
	_iterator->_next->_num = x;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorGB
// Parameters:	(void)
//
// Returns _gb of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

DScrollWidget* DSymbolVisList::iteratorGB(void)
{
  if (_iterator)
	return _iterator->_gb;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorPW
// Parameters:	(void)
//
// Returns _pw of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

DPaintWindow* DSymbolVisList::iteratorPW(void)
{
  if (_iterator)
	return _iterator->_pw;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorWhichList
// Parameters:	(void)
//
// Returns _wl of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

DWhichList* DSymbolVisList::iteratorWhichList(void)
{
  if (_iterator)
	return _iterator->_wl;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorViewIndex
// Parameters:	(void)
//
// Returns _viewIndex of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::iteratorViewIndex(void)
{
  if (_iterator)
	return _iterator->_viewIndex;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextViewIndex
// Parameters:	(void)
//
// Returns _viewIndex of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::iteratorNextViewIndex(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_viewIndex;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: deleteNext
// Parameters:	(void)
//
// Deletes the next node that the _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::deleteNext(void)
{
  if ((_iterator) && (_iterator->_next))
    {
      DSymbolVisNode * tmpnode = _iterator->_next;
      _iterator->_next=tmpnode->_next;
      if (!(tmpnode->_next))
	_tail = _iterator;
      delete tmpnode; tmpnode = NULL; 
    }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: moveIterator
// Parameters:	(void)
//
// make _iterator point to the next value
//         								    
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::moveIterator()
{
  if (_iterator)
    _iterator = _iterator->_next;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextValue
// Parameters:	(void)
//
// Point _iterator to the next node and returns the _value of the next node
// after that.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

char* DSymbolVisList::getNextValue()
{
  char* return_val;
  if ((_iterator) && (_iterator->_next) && (_iterator->_next->_next))
	{
	   _iterator = _iterator->_next;
	  return_val = _iterator->_next->_value;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: find
// Parameters:	(const char*)
//
// Points _iterator to the previous node from the node that has the given
// value, and returns 1 if that value was found.
// If the node that has the given value is the _head of the list, the it
// points _iterator to the head and returns 2.
// If the given value is not in the list, _iterator points to NULL and
// 0 is returned.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DSymbolVisList::find(const char* value)
{
  initIterator();
  char* temp;
  if (_iterator)
    if (strcmp(value,iteratorValue()) == 0)
      return 2;
    else {
      if (_iterator->_next) {
	if (strcmp(value,iteratorNextValue()) == 0)
	  return 1;
	else {
	  while (temp = getNextValue())
	    if (strcmp(value,temp) == 0)
	      return 1;
	  return 0;
	}
      }
      else 
	return 0;
    }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DSymbolVisList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DSymbolVisList::DSymbolVisList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~DSymbolVisList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DSymbolVisList::~DSymbolVisList()
{
  DSymbolVisNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only. Displays the contents of the entire list.
//         								    
//////////////////////////////////////////////////////////////////////////////
void DSymbolVisList::display()
{
  cout << "DISPLAYING LIST" << endl;
  initIterator();
  if (_iterator)
    _iterator->display();
  //if (_iterator->_next)
  //  _iterator->_next->display();
  
  for(;;)
    if (getNextValue())
      _iterator->display();
    else
      break;
  
  if ((_iterator) && (_iterator->_next))
    _iterator->_next->display();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: delete_marked_nodes
// Parameters:	(void)
//
// delete all nodes with _num of -1 in list
//         								    
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::delete_marked_nodes(void)
{
  initIterator();
  while ((_iterator) && (_iterator->_num == -1)) {
    pop();
    initIterator();
  }
  if (_iterator) {
    while (_iterator->_next) {
      if (iteratorNextNum() == -1)
	deleteNext();
      else
	_iterator = _iterator->_next;
    }
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: delete_all_nodes
// Parameters:	(void)
//
// delete all nodes in list
//         								    
//////////////////////////////////////////////////////////////////////////////

void DSymbolVisList::delete_all_nodes(void)
{
  initIterator();
  while (_iterator) {
    pop();
    initIterator();
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findCorrespondingSym
// Parameters:	(int)
//
// Returns the symbol with corresponding to the integer input x
// If the list was maintained correctly, there should be only one such symbol
// Otherwise, the first such symbol is returned
//         								    
//////////////////////////////////////////////////////////////////////////////

char* DSymbolVisList::findCorrespondingSym(int x)
{
  initIterator();
  while (_iterator) {
    if (iteratorNum() == x)
      return iteratorValue();
    else
    _iterator = _iterator->_next;
  }
}

#endif

