//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: DEWhichList.C
//  PROJ: COKO DEBUGGER
//  DATE: WINTER 2002                                                      
//
// SYNOPSIS
//  Classes for Which List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _WHICHLIST_C_
#define _WHICHLIST_C_

#include "DEWhichList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DWhichNode
// Parameters:	(int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DWhichNode::DWhichNode(int which):	
  _next(NULL)
{
  _which = which;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~DWhichNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
DWhichNode::~DWhichNode(void) 
{
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void DWhichNode::display(void) 
{
    cout << _which  << endl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (int)	
//
// Insert a new node with _which = which to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DWhichList::insert(int which) 
{
  if (!_head)
	_head = _tail = new DWhichNode(which); 
  else 
	{
	  DWhichNode * tmp = new DWhichNode(which);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: push
// Parameters: (int)	
//
// Push a new node with _which = which to the top of the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DWhichList::push(int which) 
{
  if (!_head)
	_head = _tail = new DWhichNode(which); 
  else 
	{
	  DWhichNode * tmp = new DWhichNode(which);
	  tmp->_next = _head;
	  _head = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DWhichList::pop(void)
{
  if (!_head)
	return 0;

  DWhichNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  delete tmpnode; tmpnode = NULL;

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (void)
//
// Return _which of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DWhichList::top(void)
{
  if (!_head)
    return 0;
  
  return _head->_which;
  
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int DWhichList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DWhichList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _which of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DWhichList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_which;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextValue
// Parameters:	(void)
//
// Point _iterator to the next node and returns the _value of that node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

int DWhichList::getNextValue()
{
  int return_val;
  if ((_iterator) && (_iterator->_next))
	{
	   _iterator = _iterator->_next;
	  return_val = _iterator->_which;
	}
  else
	return_val = 0;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DWhichList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DWhichList::DWhichList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~DWhichList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DWhichList::~DWhichList()
{
  DWhichNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: moveIterator
// Parameters:	(void)
//
// make _iterator point to the next value
//         								    
//////////////////////////////////////////////////////////////////////////////

void DWhichList::moveIterator()
{
  if (_iterator)
    _iterator = _iterator->_next;
}

#endif

