//////////////////////////////////////////////////////////////////
//
//  NAME: Kee-Eung Kim (kek), Joon S. Lee (jl)
//  
//  FILE: PCChildPattern.C
//  PROJ: COKO
//  DATE: 5/5/97
//  
//  SYNOPSIS
//   Method definitions for CChildPattern class
//
/////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include "KGlobals.H"
#include "KOBAnd.H"
#include "KOBEqu.H"
#include "KOBGE.H"
#include "KOBGT.H"
#include "KOBInvoke.H"
#include "KOBLE.H"
#include "KOBLT.H"
#include "KOBName.H"
#include "KOBNot.H"
#include "KOBOr.H"
#include "KOBPrimitive.H"
#include "KOBVar.H"
#include "KOBool.H"
#include "KOF1.H"
#include "KOF2.H"
#include "KOFAbs.H"
#include "KOFAdd.H"
#include "KOFAvg.H"
#include "KOFCElt.H"
#include "KOFChoose.H"
#include "KOFCompose.H"
#include "KOFCon.H"
#include "KOFConst.H"
#include "KOFCount.H"
#include "KOFDif.H"
#include "KOFDiv.H"
#include "KOFFlatten.H"
#include "KOFFold.H"
#include "KOFID.H"
#include "KOFIns.H"
#include "KOFInt.H"
#include "KOFInverse.H"
#include "KOFIter.H"
#include "KOFIterate.H"
#include "KOFJoin.H"
#include "KOFLSJoin.H"
#include "KOFMax.H"
#include "KOFMin.H"
#include "KOFMinus.H"
#include "KOFMod.H"
#include "KOFMult.H"
#include "KOFNJoin.H"
#include "KOFPCon.H"
#include "KOFPFold.H"
#include "KOFPlus.H"
#include "KOFRSJoin.H"
#include "KOFSet.H"
#include "KOFSingleton.H"
#include "KOFSort.H"
#include "KOFSum.H"
#include "KOFTimes.H"
#include "KOFUni.H"
#include "KOFUnnest.H"
#include "KOFUser.H"
#include "KOFVar.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOAny.H"
#include "KOODif.H"
#include "KOODiv.H"
#include "KOOFloat.H"
#include "KOOId.H"
#include "KOOIf.H"
#include "KOOInt.H"
#include "KOOInter.H"
#include "KOOInvoke.H"
#include "KOOLink.H"
#include "KOOMinus.H"
#include "KOOMult.H"
#include "KOOName.H"
#include "KOOPair.H"
#include "KOOPlus.H"
#include "KOOSet.H"
#include "KOOStr.H"
#include "KOOTimes.H"
#include "KOOUni.H"
#include "KOOVar.H"
#include "KOObjList.H"
#include "KOObject.H"
#include "KOPCEX.H"
#include "KOPCElt.H"
#include "KOPCFA.H"
#include "KOPCombine.H"
#include "KOPConjunct.H"
#include "KOPConst.H"
#include "KOPDisjunct.H"
#include "KOPEqual.H"
#include "KOPExists.H"
#include "KOPForall.H"
#include "KOPGreater.H"
#include "KOPGreaterEqual.H"
#include "KOPInv.H"
#include "KOPIsnotnull.H"
#include "KOPIsnull.H"
#include "KOPLess.H"
#include "KOPLessEqual.H"
#include "KOPLex.H"
#include "KOPNegate.H"
#include "KOPNotEqual.H"
#include "KOPTimes.H"
#include "KOPUser.H"
#include "KOPVar.H"
#include "KOPredicate.H"
#include "KOTWrap.H"

#include "PCChildPattern.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CChildPattern
// Parameters: (NonType*)
//
// Constructor
//         								    
//////////////////////////////////////////////////////////////////////////////

CChildPattern::CChildPattern(NonType *nt) : CPattern(nt)
{
  curr_which = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initChildMatch
// Parameters: (CState*)
//
// Initializing curr_which
// Call it before match. 
//         								    
//////////////////////////////////////////////////////////////////////////////

CState * CChildPattern::initChildMatch(CState *s)
{
  curr_which = 0;

  s->SetSuccess(T);

  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: nextChildMatch
// Parameters: (CState*)
//
// Get next matching child 
//         								    
//////////////////////////////////////////////////////////////////////////////

CState * CChildPattern::nextChildMatch(CState *s)
{
  NonType * old_root = s->Store()->Copy_kTree();
  NonType * curr_child = NULL;

  curr_which++;

  old_root->initCurrChild();

  for (int i = 0; i < curr_which; i++) 
	{
	  curr_child = (old_root->nextChild());
	  //	  if (curr_child) curr_child = curr_child->Copy_kTree();
	}

  if (curr_child) 
	{
	  // If we aren't out of boundary in the number of children,
	  // set local root to be the child and try match on it.
	  // If success, store the information. Otherwise we do it again
	  // with next child.
	  s->SetStore(curr_child->Copy_kTree());
	  s = CPattern::match(s);
	  if (s->ISuccess()) 
		{
		  CIEnvValueType * childEntry = s->IFirst();
		  s->IModify((char *)childEntry->GetKey(),
					 childEntry->GetContent()->Child_Look_Up(),
					 curr_which);
		}
	  else
		{
		  s->SetStore(old_root->Copy_kTree());
		  s = this->nextChildMatch(s);
		}
	}
  else s->SetSuccess(F);

  // Restore the old local root
  s->SetStore(old_root);

  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: build
// Parameters: (CState*)
//
// Overloaded build function
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CChildPattern::build(CState* s)
{
  NonType * root = s->Store()->Copy_kTree();
  CIdent * newChildIdent = s->IFirst()->GetContent();
  NameUnion * rootName = root->Myname();

  NonType * children[MAX_CHILDREN];
  NonType * currChild;
  NonType * newRoot;
  int i;

  // Fetch the children or the root node. (Possibly out of date)
  for(i = 0, root->initCurrChild(), currChild = root->nextChild();
	  currChild; currChild = root->nextChild(), i++)
	children[i] = currChild;

  // The updated child we want to refer when we reconstruct.
  children[newChildIdent->Which() - 1] = newChildIdent->Child_Look_Up();

  // Create appropriate new root depending on old root.
  switch(rootName->tag) {
  case FNAME:
	switch(rootName->fn) {
	case FUN_CHOOSE:
	  newRoot = new FChoose();
	  break;
	case FUN_USER:
	  newRoot = new FUser((char *) root->Name());
	  break;
	case FUN_ID:
	  newRoot = new FID();
	  break;
	case FUN_1:
	  newRoot = new F1();
	  break;
	case FUN_2:
	  newRoot = new F2();
	  break;
	case FUN_FLATTEN:
	  newRoot = new FFlatten();
	  break;
	case FUN_SET:
	  newRoot = new FSet();
	  break;
	case FUN_SINGLETON:
	  newRoot = new FSingleton();
	  break;
	case FUN_CELT:
	  newRoot = new FCElt((Function *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case FUN_INT:
	  newRoot = new FInt((Predicate *) children[0]->Copy_kTree());
	  break;
	case FUN_DIF:
	  newRoot = new FDif((Predicate *) children[0]->Copy_kTree());
	  break;
	case FUN_MOD:
	  newRoot = new FMod();
	  break;
	case FUN_ABS:
	  newRoot = new FAbs();
	  break;
	case FUN_INVERSE:
	  newRoot = new FInverse();
	  break;
	case FUN_UNI:
	  newRoot = new FUni((Predicate *) children[0]->Copy_kTree());
	  break;
	case FUN_COMPOSE:
	  newRoot = new FCompose((Function *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_TIMES:
	  newRoot = new FTimes((Function *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_CONST:
	  newRoot = new FConst((koObject *) children[0]->Copy_kTree());
	  break;
    case FUN_NJOIN:
      newRoot = new FNJoin((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree(), (Function *) children[2]->Copy_kTree());
	  break;
	case FUN_ADD:
	  newRoot = new FAdd();
	  break;
	case FUN_MULT:
	  newRoot = new FMult();
	  break;
	case FUN_DIV:
	  newRoot = new FDiv();
	  break;
	case FUN_MINUS:
	  newRoot = new FMinus();
	  break;
	case FUN_PLUS:
	  newRoot = new FPlus((Function *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_UNNEST:
	  newRoot = new FUnnest((Function *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_JOIN:
	  newRoot = new FJoin((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_PFOLD:
	  newRoot = new FPFold((Function*) children[0]->Copy_kTree(), (Function*) children[1]->Copy_kTree()); 
	  break;
	case FUN_CON:
	  newRoot = new FCon((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree(),
						 (Function *) children[2]->Copy_kTree());
	  break;
	case FUN_FOLD:
	  newRoot= new FFold((koObject *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree(),
						 (Function *) children[2]->Copy_kTree());
	  break;
	case FUN_ITERATE:
	  newRoot = new FIterate((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_ITER:
	  newRoot = new FIter((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case FUN_PCON:
	  newRoot = new FPCon((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	default: exit(1);
	}
	break;
  case PNAME:
	switch(rootName->pn) {
	case PRED_EQUAL:
	  newRoot = new PEqual();
	  break;
	case PRED_USER:
	  newRoot = new PUser((char *) root->Name());
	  break;
	case PRED_LESS:
	  newRoot = new PLess();
	  break;
	case PRED_INV:
	  newRoot = new PInv((Predicate *) children[0]->Copy_kTree());
	  break;
	case PRED_NEGATE:
	  newRoot = new PNegate((Predicate *) children[0]->Copy_kTree());
	  break;
	case PRED_FORALL:
	  newRoot = new PForall((Predicate *) children[0]->Copy_kTree());
	  break;
	case PRED_EXISTS:
	  newRoot = new PExists((Predicate *) children[0]->Copy_kTree());
	  break;
	case PRED_CELT:
	  newRoot = new PCElt((Predicate *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case PRED_DISJUNCT:
	  newRoot = new PDisjunct((Predicate *) children[0]->Copy_kTree(), (Predicate *) children[1]->Copy_kTree());
	  break;
	case PRED_CONJUNCT:
	  newRoot = new PConjunct((Predicate *) children[0]->Copy_kTree(), (Predicate *) children[1]->Copy_kTree());
	  break;
	case PRED_TIMES:
	  newRoot = new PTimes((Predicate *) children[0]->Copy_kTree(), (Predicate *) children[1]->Copy_kTree());
	  break;
	case PRED_CONST:
	  newRoot = new PConst((koBool *) children[0]->Copy_kTree());
	  break;
	case PRED_GREATER:
	  newRoot = new PGreater();
	  break;
	case PRED_LESSEQ:
	  newRoot = new PLessEqual();
	  break;
	case PRED_GREATEREQ:
	  newRoot = new PGreaterEqual();
	  break;
	case PRED_NOTEQ:
	  newRoot = new PNotEqual();
	  break;
	case PRED_COMBINE:
	  newRoot = new PCombine((Predicate *) children[0]->Copy_kTree(), (Function *) children[1]->Copy_kTree());
	  break;
	case PRED_CEX:
	  newRoot = new PCEX((Predicate *) children[0]->Copy_kTree());
	  break;
	case PRED_CFA:
	  newRoot = new PCFA((Predicate *) children[0]->Copy_kTree());
	  break;
	default: exit(1);
	}
	break;
  case ONAME:
	switch(rootName->on) {
	case OBJ_ANY:
	  newRoot = new OAny();
	  break;
	case OBJ_DIF:
	  newRoot = new ODif((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_DIV:
	  newRoot = new ODiv((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_IF:
	  newRoot = new OIf((koBool *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree(), 
						(koObject *) children[2]->Copy_kTree());
	  break;
	case OBJ_INT:
	  {
		char val[32];
		sprintf(val, "%d", ((OInt *)root)->Val());
		newRoot = new OInt(val);
		break;
	  }
	case OBJ_INTER:
	  newRoot = new OInter((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_INVOKE:
	  newRoot = new OInvoke((Function *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_MINUS:
	  newRoot = new OMinus((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_MULT:
	  newRoot = new OMult((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_NAME:
	  newRoot = new OName((char *) root->Name());
	  break;
	case OBJ_PAIR:
	  newRoot = new OPair((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_PLUS:
	  newRoot = new OPlus((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_SET:
	  newRoot = new OSet(*((OSet *) root));
	  break;
	case OBJ_STR:
	  newRoot = new OStr(((OStr *)root)->Val());
	  break;
	case OBJ_TIMES:
	  newRoot = new OTimes((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case OBJ_UNI:
	  newRoot = new OUni((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	default: exit(1);
	}
	break;
  case BNAME:
	switch(rootName->bn) {
	case BOOL_AND:
	  newRoot = new BAnd((koBool *) children[0]->Copy_kTree(), (koBool *) children[1]->Copy_kTree());
	  break;
	case BOOL_EQU:
	  newRoot = new BEqu((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_GE:
	  newRoot = new BGE((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_GT:
	  newRoot = new BGT((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_INVOKE:
	  newRoot = new BInvoke((Predicate *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_LE:
	  newRoot = new BLE((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_LT:
	  newRoot = new BLT((koObject *) children[0]->Copy_kTree(), (koObject *) children[1]->Copy_kTree());
	  break;
	case BOOL_NAME:
	  newRoot = new BName((char *) root->Name());
	  break;
	case BOOL_NOT:
	  newRoot = new BNot((koBool *) children[0]->Copy_kTree());
	  break;
	case BOOL_OR:
	  newRoot = new BOr((koBool *) children[0]->Copy_kTree(), (koBool *) children[1]->Copy_kTree());
	  break;
	case BOOL_PRIM:
	  newRoot = new BPrimitive(((BPrimitive *)root)->Value());
	  break;
	default: exit(1);
	}
	break;
  default: exit(1);
  }

  //ADDED BY DJA WINTER 2002
  if (root)
    newRoot->setVariableNum(root->getVariableNum());
  
  s->SetStore(newRoot);

  if (root) { delete root; root = NULL; } //added 4/28/97 by jl
  delete rootName; //added 3/3/97 by jl
  return s;
}

