//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk  Lee (jl)                                             
//  FILE: PCOQPrintStmt.C
//  PROJ: COKO
//  DATE: 7/98                                                   
//
// SYNOPSIS
// 
//  This class is a derived class from CStmt. It is used for coko language 
//  debugging purposes.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCOQPrintStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: COQPrintStmt
// Parameters: (int, char *)       
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

COQPrintStmt::COQPrintStmt(int which_print, char *iname):
  _iname(NULL)
{
  _which_print = which_print;
  if (iname)
  	{
  	  _iname = new char[strlen(iname)+1];
  	  strcpy(_iname, iname);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~COQPrintStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

COQPrintStmt::~COQPrintStmt(void)
{
  if (_iname) { delete [] _iname; _iname = NULL; }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *COQPrintStmt::Exec(CState *s)
{
  initstats ();
  if (_which_print == 1)
	{
	  CIdent * myNode;
	  if (_iname) 
		{ 
		  char * res = NULL;;
		  myNode = s->IFind(_iname);
		  koObject* tree = ((koObject*)(myNode->Child_Look_Up()))->obj();
		  res = tree->oql();
		  cout << res << endl;
		  if (tree) { delete tree; tree = NULL; }
		  if (res) { delete [] res; res = NULL; }		  
		}
	  else
		{
		  cout << "Warning @ PCOQPrintStmt : Exec" << endl;
		  cout << "Nothing to print out warning" << endl;
		}
	}
  else if (_which_print == 2)
	{
	  CIdent * myNode;
	  if (_iname) 
		{ 
		  char * res = NULL;;
		  myNode = s->IFind(_iname);
		  koBool* tree = ((koBool*)(myNode->Child_Look_Up()))->boo();
		  res = tree->oql();
		  cout << res << endl;
		  if (tree) { delete tree; tree = NULL; }
		  if (res) { delete [] res; res = NULL; }		  
		}
	  else
		{
		  cout << "Warning @ PCOQPrintStmt : Exec" << endl;
		  cout << "Nothing to print out warning" << endl;
		}
	}
  else if (_which_print == 3)
	{
	  NonType* tree = s->Store();
	  NameUnion* name_ = tree->Myname();
	  if (name_->tag == BNAME )
		{
		  char * res = NULL;;
		  koBool* tree2 = ((koBool*)tree)->boo();
		  res = tree2->oql();
		  cout << res << endl;
		  if (tree2) { delete tree2; tree2 = NULL; }
		  if (res) { delete [] res; res = NULL; }			  
		}
	  else if (name_->tag == ONAME)
		{
		  char * res = NULL;;
		  koObject* tree2 = ((koObject*)tree)->obj();
		  res = tree2->oql();
		  cout << res << endl;
		  if (tree2) { delete tree2; tree2 = NULL; }
		  if (res) { delete [] res; res = NULL; }	
		}
	  else
		cout << "Error @ PCOQPrintStmt.C : Current tree must be either object or bool" << endl;
	  delete name_;
	}

  s->SetSuccess(1);
  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: do_indent
// Parameters: (char*, char*, int)
//
// Add returns and indentation to OQL statement to make it readable
//         								    
//////////////////////////////////////////////////////////////////////////////

char* COQPrintStmt::do_indent(char* beg, char* end, int indent)
{
  //int count = 2;
  //int parencount = 1;
  //for (int i = 2; i < strlen(end); i++)
    
/*
  char* tempIndent = new char[indent+1];
  for (int i = 0; i < indent; i++)
    tempIndent[i] = ' ';
  tempIndent[indent] = '\0';
  strcat(beg, tempIndent);
  strcat
  cout << strstr(end, "from") << endl << endl;
  beg = strstr(end+1, "select");
  cout << beg << endl << endl;
  cout << end + 0 << endl << endl;
  cout << end + 1 << endl << endl;
  cout << end + 2 << endl << endl;
  cout << end + 3 << endl << endl;
  char* tempIndent = new char[indent+1];
  for (int i = 0; i < indent; i++)
    tempIndent[i] = ' ';
  tempIndent[indent] = '\0';
  strcat(strcat(beg, "\n"), tempIndent);
  cout << ":" << beg << ":" << endl << endl;
 */
  return end;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *COQPrintStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    if (db->really_execute()) {
      initstats ();
      if (_which_print == 1)
	{
	  CIdent * myNode;
	  if (_iname) 
	    { 
	      char * res = NULL;;
	      myNode = s->IFind(_iname);
	      koObject* tree = ((koObject*)(myNode->Child_Look_Up()))->obj();
	      res = do_indent(NULL, tree->oql(), 0);
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree) { delete tree; tree = NULL; }
	      if (res) { delete [] res; res = NULL; }		  
	    }
	  else
	    {
	      db->printInRightPlace("Warning @ PCOQPrintStmt : ExecDebug");
	      db->printInRightPlace("\n");
	      db->printInRightPlace("Nothing to print out warning");
	      db->printInRightPlace("\n");
	    }
	}
      else if (_which_print == 2)
	{
	  CIdent * myNode;
	  if (_iname) 
	    { 
	      char * res = NULL;;
	      myNode = s->IFind(_iname);
	      koBool* tree = ((koBool*)(myNode->Child_Look_Up()))->boo();
	      res = do_indent(NULL, tree->oql(), 0);
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree) { delete tree; tree = NULL; }
	      if (res) { delete [] res; res = NULL; }		  
	    }
	  else
	    {
	      db->printInRightPlace("Warning @ PCOQPrintStmt : ExecDebug");
	      db->printInRightPlace("\n");
	      db->printInRightPlace("Nothing to print out warning");
	      db->printInRightPlace("\n");
	    }
	}
      else if (_which_print == 3)
	{
	  NonType* tree = s->Store();
	  NameUnion* name_ = tree->Myname();
	  if (name_->tag == BNAME )
	    {
	      char * res = NULL;;
	      koBool* tree2 = ((koBool*)tree)->boo();
	      res = do_indent(NULL, tree2->oql(), 0);
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree2) { delete tree2; tree2 = NULL; }
	      if (res) { delete [] res; res = NULL; }			  
	    }
	  else if (name_->tag == ONAME)
	    {
	      char * res = NULL;;
	      koObject* tree2 = ((koObject*)tree)->obj();
	      res = do_indent(NULL, tree2->oql(), 0);
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree2) { delete tree2; tree2 = NULL; }
	      if (res) { delete [] res; res = NULL; }	
	    }
	  else 
	    {
	      db->printInRightPlace("Error @ PCOQPrintStmt.C : Current tree must be either object or bool");
	      db->printInRightPlace("\n");
	    }
	  delete name_;
	}
      
      s->SetSuccess(1);
      db->check_for_condition(s, rep_string(), key_number);
    }    
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* COQPrintStmt::rep_string ()
{ 
  if ((_which_print == 1) || (_which_print == 2)){
    char *res = new char [strlen (_iname) + strlen ("PrintOQL ") + 1];
    sprintf (res, "PrintOQL %s", _iname);
    return res;
  } 
  else if (_which_print == 3) {
    char *res = new char [strlen ("PrintOQL CURRTREE") + 1];
    sprintf (res, "PrintOQL CURRTREE");
    return res;
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* COQPrintStmt::representation (int indent) {
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* COQPrintStmt::representationWDI (int indent, DDebugController *db)
{
  char* temp = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp));
  return temp;
}

int COQPrintStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  return 1;
}
