/*
 * Copyright 1995, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.


//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl@cs.brown.edu)
//  FILE: PCOKO.l 
//  PROJ: COKO Compiler
//  DATE: 8/97
//
//
//////////////////////////////////////////////////////////////////////////////

*/

%{
#include "PParser.h"
#include "PCoko.tab.h"
#include <string.h>


int lineno = 1; 

%}


%%
[ \t]+                                                            { }
[\n\f]                                                            { lineno++; }
[+-]?[0-9]+                                                       { CKlval.sval = strdup(yytext); 
                                                                    return INTCON; }
\"[^\"]*[\"]                                                  	  { CKlval.sval = strdup(yytext); 
                                                                    return STRINGCON; }
[+-]?(([0-9]+)|([0-9]*\.[0-9]+)([eE][-+]?[0-9]+)?)                { CKlval.sval = strdup(yytext);
                                                                    return FLOATCON; }

[Oo][Ii][Dd]                                                      { return OID; }
"{"                                                               { return ('{'); }  
"}"                                                               { return ('}'); }     
"{|"                                                              { return DPO; }   
"|}"                                                              { return DPC; }   
[Ss][Tt][Rr]                                                      { return KSTRING; }
[Rr][Ee][Aa][Ll]                                                  { return KFLOAT; }
[Cc][Hh][Aa][Rr]                                                  { return KCHAR;}

[Pp][Rr][Oo][Pp][Ee][Rr][Tt][Yy]                                  { return PROPERTY; }
"==>"                                                             { return IMPL; }
[Ii][Nn][Ff][Ee][Rr][Ss]                                          { return INFERS; }
[-][-][^>^\n]+                                                    { } 
"."                                                               { return PERIOD; }
"<"                                                               { return LANGLE; }
">"                                                               { return RANGLE; }
"="                                                               { return EQUAL; }  
[Ee][Qq][Uu][Aa][Ll]                                              { return EQUAL; }  
[Ee][Qq]                                                          { return EQUAL; }  
[Nn][Ee][Qq]                                                      { return NOTEQUAL; }  
[Ll][Ee][Ss][Ss]                                                  { return LT; }  
[Mm][Oo][Rr][Ee]                                                  { return GT; }
"<="                                                              { return LE;}
[Ll][Ee][Qq]                                                      { return LE; } 
[Ll][Ee][Ss][Ss][Ee][Qq]                                          { return LE;} 
">="                                                              { return GE;}
[Mm][Oo][Rr][Ee][Ee][Qq]                                          { return GE;}
[Gg][Ee][Qq]                                                      { return GE; } 
"!="                                                              { return NE; } 

[Oo][Pp][Ll][Uu][Ss]                                              { return OPLUS; }      
[Oo][Ii][Nn][Tt][Ee][Rr]                                          { return INTER; } 
"\\I"                                                             { return INTER; }   
"-"                                                               { return ODIF; }  
"K"                                                               { return CONST; }  
"C"                                                               { return CURRY; }  
[Ii][Nn][Vv]      						                          { return INV; }  
[Ii][Nn][Vv][Ee][Rr][Ss][Ee]			                          { return INVERSE; }  
[Tt][Ii][Mm][Ee][Ss]						                      { return TIMES; }  
[X] 								                              { return XTIMES; }
[Ee][Xx][Ii][Ss][Tt][Ss]  	       	       	     	       	      { return EXISTS; }  
[Ff][Oo][Rr][Aa][Ll][Ll]      	      	   	     	       	      { return FORALL; }  
[Ee][Xx]            					                          { return EX; }   
[Ff][Aa]	           					                          { return FA; }   
[Ii][Dd]                             		                      { return ID; }  
[Tt][Ii][Dd]                             		                  { return TID; }  
[Ss][Ee][Tt]                                                      { return SET; }  
[#][1]                             				                  { return ONE; }  
[#][2]                             				                  { return TWO; }  
[Aa][Vv][Gg]                                                      { return AVG; }
[Ss][Uu][Mm]                                                      { return SUM; }
[Mm][Ii][Nn]                                                      { return MIN; }
[Mm][Aa][Xx]                                                      { return MAX; }
[Cc][Oo][Uu][Nn][Tt]                                              { return COUNT; }
[Ll][Ee][Xx]                                                      { return LEX; }
[Aa][Dd][Dd]                                                      { return ADD; }
[Ss][Uu][Bb]                                                      { return SUB; }
[Mm][Uu][Ll]                                                      { return MUL; }
[Dd][Ii][Vv]                                                      { return DIV; }
[Ss][Ii][Nn][Gg][Ll][Ee][Tt][Oo][Nn]                              { return SINGLETON; }
[Ss][Hh][Ll]                					                  { return SHIFTL; }  
[Ss][Hh][Rr]			                    		              { return SHIFTR; }  
[Cc][Hh][Oo][Oo][Ss][Ee]                                          { return CHOOSE; }  
[Ff][Ll][Aa][Tt][Tt][Ee][Nn]                                      { return FLATTEN; }  
"o"					                                              { return CIRC; }  
"+"                                                               { return PLUS; }  
[Cc][Oo][Nn]           						                      { return CON; } 
[Uu][Nn][Ii]                                                      { return UNI; }  
[Ii][Nn][Tt]                                                      { return INT; }  
[Dd][Ii][Ff]                                                      { return DIF; }  
[Ff][Oo][Ll][Dd]                                                  { return FOLD; }  
[Ii][Tt][Ee][Rr][Aa][Tt][Ee]                                      { return ITERATE; }  
[Ii][Tt][Ee][Rr]                                                  { return ITER; }  
[Pp][Cc][Oo][Nn]                                                  { return PCON; }  
[Jj][Oo][Ii][Nn]                                                  { return JOIN; }  
[Nn][Jj][Oo][Ii][Nn]                                              { return NJOIN; }
[Uu][Nn][Nn][Ee][Ss][Tt]                                          { return UNNEST; }  
[Pp][Ff][Oo][Ll][Dd]                                              { return PFOLD; }
[Ee][Mm][Pp][Tt][Yy]                                              { return EMPTY; }  
[Ss][Oo][Rr][Tt]                                                  { return SORT; }
[Ii][Nn][Ss]                                                      { return INS; }
[Ll][Ss][Jj][Oo][Ii][Nn]                                          { return LSJOIN; }
[Rr][Ss][Jj][Oo][Ii][Nn]                                          { return RSJOIN; }

[Nn][Ii][Ll]                                                      { return NIL; }  
[Ff][Aa][Ll][Ss][Ee]                                              { return FALSEv; }  
[F]                                                               { return FALSEv; }  
[Tt][Rr][Uu][Ee]                                                  { return TRUEv; }  
[T]                                                               { return TRUEv; }  
[Bb][Ee][Gg][Ii][Nn]                                              { return BEG; }  
[Ee][Nn][Dd]                                                      { return END; }  
"!"								                                  { return FINVOKE; }
"?"								                                  { return PINVOKE; }

","                                                               { return (','); }  
":"                                                               { return (':'); }  
";"                                                               { return (';'); }  
"|"                                                               { return ('|'); }  
"\\/"                                                             { return OR; }
"/\\"                                                             { return AND; }
[Nn][Oo][Tt]                                                      { return NOT; }
"("                                                               { return ('('); }  
")"                                                               { return (')'); }  
"["                                                               { return ('['); }  
"]"                                                               { return (']'); }  
"&"                                                               { return ('&'); }  
"~"            						                              { return ('~'); }  
[_][Oo]            						                          { return ODC; }  
[_][Ff]            						                          { return FDC; }  
[_][Pp]            						                          { return PDC; }  
[_][Bb]            						                          { return BDC; }  
[_][Ii]            						                          { return IDC; }  
[_][Tt]            						                          { return TDC; }  
[pqr][0-9']*                                                      { CKlval.sval = strdup(yytext); 
                                                                    return PIDENT; }
(op)[0-9']*                                                       { CKlval.sval = strdup(yytext); 
                                                                    return PIDENT; }
[fghjk][0-9']*                                                    { CKlval.sval = strdup(yytext); 
                                                                    return FIDENT; }
[xyzABC][0-9']*                                                   { CKlval.sval = strdup(yytext); 
                                                                    return OIDENT; }
[b][0-9']*                                                        { CKlval.sval = strdup(yytext); 
                                                                    return BIDENT; }
[t][0-9']*                                                        { CKlval.sval = strdup(yytext); 
                                                                    return TIDENT; }
[a-zA-Z][a-z_A-Z0-9]*[\*]*                  			          { CKlval.sval = strdup(yytext); return IDENT; }
.                    						                      { return ERROR; }
%%



int yywrap ()  {return 1;}

