/*
 * Copyright 1995, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/*
//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl@cs.brown.edu)
//  FILE: PParser.C
//
//  PROJ: COKO Compiler
//  DATE: 8/97                                                          
//
//////////////////////////////////////////////////////////////////////////////
*/

/* -*-Mode:c++-*- */

#include "PParser.h"

PPropertyBlock::PPropertyBlock(string name, PAIdentSet includes, PCode *body)
{
  _name = name;
  _bodyCode = body;
  _includes = includes;
}

const char *PPropertyBlock::Name(void)
{
  return _name.chars();
}

void PPropertyBlock::Emit(ofstream & foutH, ofstream & foutC)
{
  PAIdentSet::iterator i;

  foutH << "#ifndef _" << _name << "Property_H" << endl;
  foutH << "#define _" << _name << "Property_H" << endl;
  foutH << endl;
  foutH << "#include \"CustomProperty.H\"" << endl;
  foutH << endl;
  foutH << "class " << _name << "Property : public CustomProperty{\n" << endl;
  foutH << "private:" << endl;
  foutH << "\t" << _name << "Property() : CustomProperty ( createProperty()) {}\n" << endl;
  foutH << "\tstatic Property *createProperty();\n" << endl;
  foutH << "public:" << endl;
  foutH << "\tstatic Property * instance();" << endl;
  foutH << "};" << endl;
  foutH << "#endif" << endl;

  foutC << "#include \"" << _name << "Property.h\"" << endl;
  foutC << "#include \"MatchProperty.H\"" << endl;
  foutC << "#include \"SchemaProperty.H\"" << endl;
  foutC << "#include \"AndProperty.H\"" << endl;
  foutC << "#include \"OrProperty.H\"" << endl;
  foutC << "#include \"NotProperty.H\"" << endl;
  foutC << "#include \"BoundProperty.H\"" << endl;

  for( i = _includes.begin(); i != _includes.end(); i++) { 
    foutC <<  *i << endl;
  }

  foutC << "Property * " << _name << "Property::createProperty(){" << endl;
  foutC << "return " << _bodyCode->code.chars() << ";" << endl;
  foutC << "}" << endl;
  foutC << endl;

  foutC << "Property *" << _name << "Property::instance(){" << endl;
  foutC << "\tstatic " << _name << "Property *theProp = 0;" << endl;
  foutC << endl;
  foutC << "\tif (theProp == 0)" << endl;
  foutC << "\t\ttheProp = new " << _name << "Property();" << endl;
  foutC<< endl;
  foutC << "return theProp;" << endl;
  foutC << "}";
}
