//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)   
//  FILE: PCRuleInvInvokeStmt.H
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
// Class definitions for CRuleInvInvokeStmt.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCRULEINVINVOKESTMT_H_
#define _PCRULEINVINVOKESTMT_H_

#include "PCStmt.H"
#include "PCRule.H"

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CRuleInvInvokeStmt
//
// SYNOPSIS
//  This class is a derived class from CStmt. It is used for rule inverse
//  invoke statements in COKO rule block.
//
//////////////////////////////////////////////////////////////////////////////

class CRuleInvInvokeStmt : public CStmt
{
protected:
  CRule *_first;          // The rule we are inverse-invoking
  char * _second;         // The identifier name we are going to invoke with

public:
  CRuleInvInvokeStmt (CRule*, char *iname);
  //  Constructor for inverse rule invokation statement.

  ~CRuleInvInvokeStmt();
  //  Destructor

  CState *Exec(CState*);
  //  Executes the given statement.
  //  Takes the state of the program before the statement
  //  is executed, and then returns the state of the 
  //  program after the statement is executed.


  //DJA SUMMER 2001:
  //Added rep_string, representation, is_leaf and ExecDebug methods
  
  char* rep_string ();
  //returns the string representation of only the current 
  //node of the COKO tree

  char* representation (int);
  //returns string representation of COKO tree

  char* representationWDI(int, DDebugController *);
  //returns string representation of COKO tree
  //but also update the debugger with the location (row and
  //column) of each breakable statement's substring

  CState *ExecDebug(CState *, DDebugController *);
  //what to do when executed in debug mode
  
  int is_leaf();
  //returns whether current statement is a leaf in the COKO tree

};

#endif
