//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek), Joon S. Lee (jl)
//
//  FILE: PCStmt.H
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Class definition for CStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCSTMT_H_
#define _PCSTMT_H_

#include <stdio.h>
#include <stdlib.h>
#include "PCState.H"
#include "PCPattern.H"
#include "DEDebugController.H"

class CState;

extern int _debug_;
extern int _cldbg_;
extern int _clcokodb_;
extern int _vcokodb_;

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CStmt
//
// SYNOPSIS
//  This class is an abstract base class for rule blocks statements. 
//
//////////////////////////////////////////////////////////////////////////////

class CStmt {
public:
  
  virtual CState *Exec(CState *s) = 0;
 // What to do when executed 

  //virtual CState *ExecDebug(CState *);

  virtual ~CStmt() {}
  // Destructor (virtual)

  virtual CState *ExecDebug(CState *, DDebugController *) = 0;

  virtual CState* updateDEP (char*, CState*, CIEnvStackDataType*, int);
  // updates variable dependancy list


  //DJA SUMMER 2001:
  //Added rep_string, representation, number_self, output_self, get_key_number, get_indent_string, is_leaf and ExecDebug methods

  virtual char* representation (int);
  //returns the string representation of the COKO subtree
  //beginning at this object

  virtual char* representationWDI (int, DDebugController*);
  //returns the string representation of the COKO subtree
  //beginning at this object but includes debugging info about string

  virtual char* rep_string ();
  //returns the string representation of only the current 
  //node of the COKO tree

  virtual int number_self(int);
  //assigns an integer to the current node for use in the
  //debugger for the breakOptions command

  virtual void output_self(DDebugController *db);
  //output key_number and rep_string for debugger
  //for breakOptions command

  virtual int get_key_number();
  //returns the value stored in the protected variable key_number;

  virtual char* get_indent_string(int indent);

  virtual int is_leaf() = 0;
  
protected:

  //ADDED DJA SUMMER 2001
  //stores the key_numer used by the debugger to refer to this statement
  //set in the number_self method
  int key_number;

};
#endif
