/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)
//  FILE: KOBPrimitive.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  The concrete type of Boolean primitives (i.e., true or
//  false).  val is the value of the primitive.
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_BPRIMITIVE_HEADER
#define KOLA_BPRIMITIVE_HEADER 

#include "KGlobals.H"
#include "KOBool.H"
#include "KONameUnion.H"

#ifdef _OSTORE_
#include "OSBPrimitive.H"
#endif // _OSTORE_

#include "PCState.H"
#include <string.h>

class BPrimitive : public koBool
{
public:
  BPrimitive (int v, TBool * t = NULL);
  // Constructor.

  BPrimitive (const BPrimitive& other);
  // Copy Constructor.
  
  ~BPrimitive ();
  // Destructor.

  koBool* Copy () const;
  // virtual Copy Constructor.
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 0; }
  //  Number of arguments.
    
  int Value () const {return val;}
  //  What is the value of this primitive?
  
  //  BResult* reduce ();
  //  return new BResCONST (val)
	  
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  Return "BPRIM: " + "TRUE" or "FALSE"

  void initCurrChild() {}
  // This is a dummy function for all the classes 
  // that don't have any child.

  NonType* nextChild() { return NULL; }
  // This is dummy functions for all the classes 
  // that don't have any child.
  // This dummy function returns NULL since there is no next child for 
  // those classes (classes w/o any child.) 

  const char * Name() const { return namestring; }
  // Return a character string representing the name of the function.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack. 
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  NonType* transform(CState* s);
  //  Return coresponding NonType object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.       

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  char *sql ();
  // Return the SQL string equivalent of this object

  char *oql ();
  // Return the OQL string equivalent of this object

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
  // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koBool* bool() { return this; }
  // Returns itself.

  OSBool* result() { return new OSBPrimitive(Value()); }
  // Returns the OSBool equivalent of the object.
#endif // _OSTORE_

protected:
  int val;
  //  What is the value of this primitive?
};


#endif

 



