/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Xiaoyu Wang(wangxy@cs.brandeis.edu)
//  FILE: KOFElt.H 
//  PROJ: COKO, KOLA
//  DATE: 10/00                                                    
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FELT_HEADER
#define KOLA_FELT_HEADER 

#include "KGlobals.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOInvoke.H"

#include "PCState.H"
#include "KOFVar.H"
#include "KOPVar.H"

#include <string.h>

class FElt : public Function
{
public:
  FElt ();
  //  Constructor.  p is the predicate argument and
  //  f1 and f2 the function arguments to the former.

  FElt (const FElt& other);
  //  Copy Constructor. 
  
  ~FElt ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 0; }
  //  Number of arguments.

  char* prolog ();
  // return string representation in prolog term
  
  void initCurrChild(){}
  //does nothing, has no child

  char* representation ();

  NonType* nextChild(){
    return NULL;
  }

  void replaceChild(NonType* child, int which_child){
  }
  // does nothing, since it has no child
  
  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 
  
  char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x);
  //  Implements the conditional function former for koObjects.

  OSObject* exec(OSObject* x);
  //  Implements the conditional function former for OSObjects.
#endif // _OSTORE_

protected:
};
# endif

