/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOFInt.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFInt.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FInt
// Parameters:	(Predicate* eq)							  
//
//  Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FInt::FInt (Predicate *eq, TFunc * ftype) : Function (ftype, FUN_INT,"INT")
{
  one = eq;
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FInt
// Parameters:	(FInt& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FInt::FInt (const FInt& other) : Function ((Function &) other)
{
  one = (other . One ()) -> Copy ();
  variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FInt
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FInt::~FInt ()
{
  delete one;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()							  
//
//  Virtual Copy Constructor 
//         								    
//////////////////////////////////////////////////////////////////////////////

Function *FInt::Copy () const
{
  FInt* temp = new FInt (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int FInt::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_INT))
  {
    delete name_;
    return (one->deepeq(((FInt*)other)->One()));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* FInt::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_INT);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* FInt::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FInt::reduce (OResult* a)
// {
//   if (! (a -> isPair ())) {
//     char *or = one -> representation ();
//     char *r = new char [strlen (or) + strlen ("INT ()") + 1];
//     sprintf (r, "INT (%s)", or);
//     OResult *res = new OResBB (r, a -> Copy ());
//     delete [] or;
//     delete [] r;
//     delete a;
//     return res;
    
//   } else {
//     OResVAR *v1 = new OResVAR ();
//     OResVAR *v2 = new OResVAR ();
//     OResPAIR *a2 = ((OResPAIR *) a);

//     OResUNISET *res = 
//       new OResUNISET
//       (new OResSET
//        (v1 -> Copy (),
// 	new BResCOMP
// 	(new BResINF (v1 -> Copy (), IN, a2 -> One () -> Copy ()), 
// 	 SAND,
// 	 new BResQUAN 
// 	 ('E', v2 -> Copy (),
// 	  new BResCOMP
// 	  (new BResINF (v2 -> Copy (), IN, a2 -> Two () -> Copy ()), 
// 	   SAND,
// 	   one -> reduce (new OResPAIR (v1 -> Copy (), v2 -> Copy ()))
// 	   )
// 	  )
// 	 )
// 	),
//        (new OResSET
// 	(v2 -> Copy (),
// 	 new BResCOMP
// 	 (new BResINF (v2 -> Copy (), IN, a2 -> Two () -> Copy ()), 
// 	  SAND,
// 	  new BResQUAN 
// 	  ('E', v1 -> Copy (),
// 	   new BResCOMP
// 	   (new BResINF (v1 -> Copy (), IN, a2 -> One () -> Copy ()), 
// 	    SAND,
// 	    one -> reduce (new OResPAIR (v1 -> Copy (), v2 -> Copy ()))
// 	    )
// 	   )
// 	  )
// 	 )
// 	)
//        );
//     delete v1;
//     delete v2;
//     delete a;
//     return res;
//   }
// }


// ////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FFO: FInt (" + one -> representation () + ")"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FInt::representation ()
{
  char *orr = one -> representation ();
  char *res = new char [strlen (orr) + strlen ("INT ()") + 1];
  sprintf (res, "INT (%s)", orr);
  delete [] orr;    
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding Nontype object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FInt::transform(CState* s)
{
  Predicate* first = (Predicate*) One() -> transform(s);
  FInt* temp = new FInt(first,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  transformDeep
// Parameters:  (CState*, CIEnvStackDataType*)
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly.
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only,
//  while TransformDeep methods searches entire stack starting from
//  a given point.
//
//////////////////////////////////////////////////////////////////////////////

NonType* FInt::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  Predicate* first = (Predicate*) One() -> transformDeep(s, startingPt);
  FInt* temp = new FInt(first,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FInt::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_INT))
	{
	  // do it for first child.
	  if (one->compares(((FInt*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((FInt*)nt,((FInt*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((PVar*)one)->Name()))->Child_Look_Up())->deepeq(((FInt*)nt)->One())))
					return_val++;
                                  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
          else return_val++;
	  // compare failed
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for statement matching only.
// Statement matching is different from rule matching.
// No duplicated variable entry is allowed here.
//
// For example,
//             match f ! _O,
//                   f =  g o g  is a illegal statement (equation).
//                       -------
//                         |
//                         |_\
//                           / matching part
//
// Assume that we have a query tree of the form "ID o ID ! 3."
// Variable f gets bounded to "ID o ID" and we have to match "ID o ID" to "g o g."
// Matching "ID o ID" to "g o g" is perfectly legal in a rule section.
// However, it is illegal in statement section of the program.
// Remember that every variable on the right hand side of an equation must be a fresh one.
// However, after we match first g to ID, the second g is no longer fresh variable.
//
// "No duplication of a variable is legal in a statement section."
//
//////////////////////////////////////////////////////////////////////////////

int FInt::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_INT))
	{
	  // do it for first child.
	  if (one->comparesSTM(((FInt*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((FInt*)nt,((FInt*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((PVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
          else return_val++;
	  // compare failed
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void FInt::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void FInt::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FInt::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:
//
//  Implements the set difference.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FInt::exec(OSObject *x)
{
  return (new LSJoinSieve(new PCEX(new PEqual),new FID,
                         (Iterator*)((OSPair*)x)->One(),
                         (Iterator*)((OSPair*)x)->Two()))->set();
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FInt::prolog ()
{
  char *orr = one -> prolog ();
  char *res = new char [strlen (orr) + strlen ("fint()") + 1];
  sprintf (res, "fint(%s)", orr);
  delete [] orr;    
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FInt::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("fint"),1,or);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FInt::sql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> sql ();
	  char *arg2 = o -> asPair () -> Two () -> sql ();
	  if (strncmp (arg1, "ERROR", 5) == 0) {
		delete [] arg2;
		return arg1;
	  }  else if (strncmp (arg2, "ERROR", 5) == 0) {
		delete [] arg1;
		return arg2;
	  } else {
		char *res = new char [strlen ("( INTERSECT )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s INTERSECT %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling INT on non-Pair Object" << endl;
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FInt::oql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> oql ();
	  char *arg2 = o -> asPair () -> Two () -> oql ();
	  if (strncmp (arg1, "ERROR", 5) == 0) {
		delete [] arg2;
		return arg1;
	  }  else if (strncmp (arg2, "ERROR", 5) == 0) {
		delete [] arg1;
		return arg2;
	  } else {
		char *res = new char [strlen ("( INTERSECT )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s INTERSECT %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling INT on non-Pair Object" << endl;
	  return NULL;
	}
}

