/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOFUser.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFUser.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FUser
// Parameters:	(char* n)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FUser::FUser (char* s, TFunc * ftype) : Function (ftype, FUN_USER, s)
{
  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  } 
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FUser
// Parameters:	(FUser& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FUser::FUser (const FUser& other) : Function ((Function &) other)
{
  if (other.val == NULL) val = NULL;
  else {
    val = new char [strlen(other.val) + 1];
    sprintf (val, "%s", other.val);
  }
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FUser
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FUser::~FUser ()
{
 if (val) {delete [] val; val = NULL;}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Function*
//         								    
//////////////////////////////////////////////////////////////////////////////

Function* FUser::Copy () const
{
  FUser* temp = new FUser (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  deepeq
// Parameters:  ()                                                        
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//                                                                          
//////////////////////////////////////////////////////////////////////////////

int FUser::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_USER))
  {
    delete name_;
    if (!strcmp(((FUser*)other)->Name(),val))
      return T;
    else
      return F; 
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  Myname
// Parameters:  ()
//
//  Returns a union type, NameUnion.
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, Index
//  and Bool, and those 5 different sub-classes use different enum type for
//  their own sub-classes' names, Myname methods needs a union type of
//  5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this
//  methods with NonType* to identify the specific instance of Kola object.
//
//////////////////////////////////////////////////////////////////////////////

NameUnion* FUser::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_USER);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)							  
//  return
//    new OBBox (name + " ! " + a -> reduce ())
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FUser::reduce (OResult* a)
// {    
//   OResBB *res = new OResBB ((char *) Name(), a -> Copy (), T);
//   delete a;
//   return res;
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FUSER: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FUser::representation ()
{
if (type)
{
  char *tmp = (type->Dom())->representation();
  char *tmp2 = (type->Ran())->representation();
  char *res = new char [strlen (Name()) + strlen ("Fun(, , )") + strlen (tmp) +
strlen(tmp2) + 1];
  sprintf (res, "Fun(%s, %s, %s)", Name(),tmp,tmp2);
  if (tmp) {delete tmp; tmp = NULL;}
  if (tmp2) {delete tmp2; tmp2 = NULL;}
  return res;
}
else
{
  char *tmp = "NULL";
  char *tmp2 = "NULL";
  char *res = new char [strlen (Name()) + strlen ("Fun(, , )") + strlen (tmp) +
strlen(tmp2) + 1];
  sprintf (res, "Fun(%s, %s, %s)", Name(),tmp,tmp2);
  return res;
}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FUser::transform(CState* s)
{
  FUser* temp = new FUser(val,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\ 
//                    |_\         / building part
//                      / matching part 
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair 
// into the env stack. Then the next time we face another g (second one), 
// we search the stack for the matching pair with variable name g and simply check 
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID). 
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FUser::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_USER) && (!strcmp(((FUser*)nt)->Name(),val)))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FUser::replaceChild(NonType* child, int which_child)
{
}  

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Executes the user-defined method specified by Name().
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FUser::exec(OSObject* x)
{
  funcptr code = NULL;
  os_class_type *class_type = ((os_class_type*)os_type::type_at(x));
  const char *t = class_type->get_name();

  while (code == NULL) {
    code = FTableLookup(t, Name());

    // Look for the class's base class (if any).  This is necessary
    // in the event that a class's method is inherited from the
    // base class' definition (virtual function is not redefined).

    if (code == NULL) {
      os_Cursor<os_base_class*>class_list(class_type->get_base_classes());
      os_base_class *b = class_list.first();

      // if there are no base classes, function name is incorrect
      if (b == 0)
        throw NoSuchFuncErr();

      os_class_type &c = b->get_class();
      class_type = &c;
      t = c.get_name();
    }
  }

  return (*code)(x);
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FUser::prolog ()
{
  char *tmp, *tmp2;
  if (type == NULL) {
	tmp = new char [strlen ("knull") + 1];
	tmp2 = new char [strlen ("knull") + 1];
	sprintf (tmp, "%s", "knull");
	sprintf (tmp2, "%s", "knull");
  } else {	
	tmp = (type->Dom())->prolog();
	tmp2 = (type->Ran())->prolog();
  };
  char *res = new char [strlen (Name()) + strlen ("fun(k, , )") + strlen (tmp) + strlen(tmp2) + 1];
  sprintf (res, "fun(k%s, %s, %s)", Name(),tmp,tmp2);
  if (tmp) {delete tmp; tmp = NULL;}
  if (tmp2) {delete tmp2; tmp2 = NULL;}
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FUser::sp_prolog ()
{
  SP_term_ref dom, ran;
  if (type != NULL) {
	dom = (type->Dom())->sp_prolog();
	ran = (type->Ran())->sp_prolog();
  } else {
	dom = SP_new_term_ref();
	ran = SP_new_term_ref();
	SP_put_string(dom,"knull");
	SP_put_string(ran,"knull");
  };

  char *tmp = new char [strlen(Name()) + 2];
  sprintf(tmp,"k%s",Name());

  SP_term_ref nam = SP_new_term_ref();
  SP_term_ref res = SP_new_term_ref();

  SP_put_string(nam,tmp);

  SP_cons_functor(res, SP_atom_from_string("fun"),3,nam,dom,ran);

  if (tmp) {delete tmp; tmp = NULL;}
 
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FUser::sql (koObject *o)
{    
  char *arg1, *arg2;

  NameUnion *nu = o -> Myname ();
  if (nu ->on == OBJ_PAIR) {
    delete nu;
	arg1 = o -> asPair () -> One () -> sql ();
	
	koObject *o2 = o -> asPair () -> Two ();
	nu = o2 -> Myname ();
	if (nu ->on == OBJ_PAIR) 
	  arg2 = o2 -> asPair () -> asargs ();
	else
	  arg2 = o2 -> sql ();
	delete nu;

    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
      return arg1;
	} else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
      return arg2;
	} else {
      char *res = new char [strlen (". (%s)") + strlen (val) + strlen (arg1) + strlen (arg2) + 1];
      sprintf (res, "%s.%s (%s)", arg1, val, arg2);
      delete [] arg1;
      delete [] arg2;
	  return res;
    }
  } else {
    delete nu;
	char *arg = o -> sql ();
	if (strncmp (arg, "ERROR", 5) == 0) {
	  return arg;
	} else {
	  char *res = new char [strlen (".") + strlen (arg) + strlen (val) + 1];
	  sprintf (res, "%s.%s", arg, val);
	  delete [] arg;
	  return res;
	}
  }	
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FUser::oql (koObject *o)
{    
  char *arg1, *arg2;

  NameUnion *nu = o -> Myname ();
  if (nu ->on == OBJ_PAIR) {
    delete nu;
	arg1 = o -> asPair () -> One () -> oql ();
	
	koObject *o2 = o -> asPair () -> Two ();
	nu = o2 -> Myname ();
	if (nu ->on == OBJ_PAIR) 
	  arg2 = o2 -> asPair () -> asargs ();
	else
	  arg2 = o2 -> oql ();
	delete nu;

    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
      return arg1;
	} else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
      return arg2;
	} else {
      char *res = new char [strlen (". (%s)") + strlen (val) + strlen (arg1) + strlen (arg2) + 1];
      sprintf (res, "%s.%s (%s)", arg1, val, arg2);
      delete [] arg1;
      delete [] arg2;
	  return res;
    }
  } else {
    delete nu;
	char *arg = o -> oql ();
	char *res = new char [strlen (".") + strlen (arg) + strlen (val) + 1];
	sprintf (res, "%s.%s", arg, val);
    delete [] arg;
	return res;
  }	
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FUser::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}

