/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)					 	    
//  FILE: NonType.H 
//  PROJ: KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  Supertype of all nontype objects, which include
//  Bools, Objects, Functions and Predicates.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_NONTYPE_HEADER
#define KOLA_NONTYPE_HEADER

#include "KGlobals.H"
#include "KONameUnion.H"

#ifdef _PROLOG_
#include "SPPROLOG.H"
#endif // _PROLOG_

#include "PCState.H"

class CState;
class CIEnvStackDataType;

class NonType
{
public:

  virtual int getVariableNum() const { return variableNum;}
  virtual void setVariableNum(int newNum) { variableNum = newNum;}

  virtual char* representation () = 0;
  // Returns a string representation of the subtree
  // beginning at this object.


  virtual char* prolog () = 0;
  // return string representation in prolog term

  virtual ~NonType() {}
  // virtual destructor

  virtual int numChildren() const  = 0;
  //  Number of arguments.

  virtual int deepeq (NonType * other) = 0;
  // Value-based equality

  virtual NameUnion* Myname() = 0;
  // returns myname wrapped in side of a class NameUnion.

  virtual  NonType* Copy_kTree() const = 0;
  // Returns a copy of kola query tree.
  // This methods was added because there was no way to make
  // a copy of given kola query from NonType level with existing methods.
  // Existing Copy() methods alone wasn't suitable for the task 
  // since return type of Copy() is different from class to class.
  // Copy_kTree() uses Copy() methods to get the task work by wrapping a 
  // Copy methods and returning NonType*.

  virtual  void initCurrChild() = 0;
  // This is a dummy virtual function for all the classes 
  // that don't have any child.
  // Any function that has at least one child must overload 
  // this virtual function.

  virtual NonType* nextChild() = 0;
  // This is dummy virtual functions for all the classes 
  // that don't have any child.
  // Any function that has at least one child must overload 
  // this virtual function.
  // This dummy function returns NULL since there is no next child for 
  // those classes (classes w/o any child.) 

  virtual void replaceChild(NonType*, int) = 0;
  // Child replacing method.
  // This method calls SetOne, SetTwo, or SetThree methods.

  virtual CState* unify(CState * a);
  // Calls compares methods on the query tree(Nontype).

  virtual CState* unifySTM(CState* a);

  virtual int compares (NonType* nt, CState* a) = 0;
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  virtual int comparesSTM(NonType* nt, CState* a) { return compares(nt, a); }
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.

  virtual int isChildVar(NonType* child);
  // Check if a given NonType(child) is variavle or not
  // This function sould not be overwritten by any other classes

  virtual NonType* transform(CState* a) = 0;
  //  Return coresponding Nontype object.
  //  Transform NonType to NonType with variables replaced with it's match
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  virtual NonType* transformDeep(CState* a, CIEnvStackDataType* startingPt) { return transform(a);}
  //  Return coresponding NonType object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
  //  However, CState is kept for making transform methods uniformly. 
  //  The difference between this method and transform method is a search
  //  method used in VAR classes.
  //  Transform method searches the top frame only, 
  //  while TransformDeep methods searches entire stack starting from 
  //  a given point. 

  virtual void setDEPnInvDEP(char*, CState*) = 0;
  // set dependency list and inverse dependency list

  virtual void undoSetInvDEP(char*, CState*) = 0;
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  virtual const char * Name() const = 0;
  // Return the character representation of the name of this node.

#ifdef _PROLOG_
  virtual SP_term_ref sp_prolog () = 0;
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_

protected:
  int variableNum;

};

#endif










