/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOAny.c
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOAny.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OAny
// Parameters:	()
//
// Constructor
//         								    
//////////////////////////////////////////////////////////////////////////////

OAny::OAny (TObj * t) : koObject (t,"NIL")
{
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OAny
// Parameters:	(Any v)							  
//
// Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OAny::OAny (const OAny& other) : koObject ((koObject &) other)
{
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OAny
// Parameters:	()							  
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OAny::~OAny ()
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OAny::Copy () const
{
  OAny* temp = new OAny (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int OAny::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_ANY))
  {
    delete name_;
    return T;
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OAny::Myname()
{
        NameUnion* a = new NameUnion(ONAME, OBJ_ANY);
        return a;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return as_string (val)
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OAny::reduce ()
// {
//   return new OResID ("NIL");
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return val as a string
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OAny::representation ()
{
  char *res = new char [strlen ("NIL") + 1];
  sprintf (res, "NIL");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OAny::transform(CState* s)
{
  OAny* temp = new OAny(Type());
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int OAny::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_ANY))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OAny::replaceChild(NonType* child, int which_child)
{
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OAny::prolog ()
{
  char *res = new char [strlen ("oany") + 1];
  sprintf (res, "oany");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OAny::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"oany");
  return res;
}
#endif // _PROLOG_


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OAny::sql ()
{
  char *res = new char [4];
  sprintf (res, "NIL");
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OAny::oql ()
{
  char *res = new char [4];
  sprintf (res, "NIL");
  return res;
}
