/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOOLink.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  Object link.  FOrm a chain of members to a set.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_OLINK_HEADER
#define KOLA_OLINK_HEADER 

#include "KGlobals.H"
#include "KOObject.H"
#include "KOObjList.H"
#include <string.h>

#include "PCState.H"

class OLink
{
public:
  OLink (koObject* o);
  //  Constructor
  
  OLink (const OLink & other);
  //  Copy Constructor
  
  ~OLink ();
  //  Destructor
  
  koObject* Obj () const {return _obj;}
  //  What object is kept at this link?
  
  void Next (OLink* o) {next = o;}
  //  Set next to o.
  
  OLink* Next () const {return next;}
  //  Selector.  What is next OLink?  (NULL if none)
	  
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  return ob -> representation ()

  char *sql ();
  // Return the SQL string equivalent of this object

  char *oql ();
  // Return the OQL string equivalent of this object

  ObjList *objlist (ObjList *ol);
  // Symbolic simplification of all objects linked to this

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

protected:
  koObject* _obj;
  //  What object is kept at this link?
  
  OLink * next;
  //  What is the next link in the chain?
  
};

# endif
