/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)					 	 
// FILE: KOPExists.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOPExists.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PExists
// Parameters:	(Predicate* p)				
//
//  Constructor.  p is predicate argument.
//         								    
//////////////////////////////////////////////////////////////////////////////

PExists::PExists (Predicate* p, TPred * t) : Predicate (t, PRED_EXISTS,"EXISTS")
{
    one = p;
    variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PExists
// Parameters:	(PExists& other)				 
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

PExists::PExists (const PExists& other) : Predicate ((Predicate &) other)
{
    one = other . One () -> Copy ();
    variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~PExists
// Parameters:	()
//
//  Destructor
//         								    
//////////////////////////////////////////////////////////////////////////////

PExists::~PExists ()
{
    delete one;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()
//
//  Return copy of *this as Predicate
//         								    
//////////////////////////////////////////////////////////////////////////////

Predicate* PExists::Copy () const
{
  PExists* temp = new PExists (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:deepeq
// Parameters:(Nontype*)
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////

int PExists::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == PNAME ) && (name_->pn == PRED_EXISTS))
  {
    delete name_;
    return (one->deepeq(((PExists*)other)->One()));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* PExists::Myname()
{
	NameUnion* a = new NameUnion(PNAME, PRED_EXISTS);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//
//   Function Name: nextChild
//   Parameters: ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* PExists::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)					   
//
//  v = OResVar ();
//  Return new BResFA 
//   (v,
//    new BResCOM
//      (new BResINF (v, "IN", a -> reduce ()),
//       "/\",
//       p -> reduce (v)
//      )
//         								    
//////////////////////////////////////////////////////////////////////////////

// BResult* PExists::reduce (OResult* a)
// {

//   OResVAR *v = new OResVAR ();
//   BResQUAN *res = 
//     new BResQUAN
//      ('E', v -> Copy (),
//       new BResCOMP
//         (new BResINF (v -> Copy (), IN, a -> Copy ()),
//          SAND,
//          one -> reduce (v -> Copy ())
// 	)
//      );

//   delete v;
//   delete a;
//   return res;

// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "PFO: EXISTS" + one -> representation ()
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PExists::representation ()
{

    char *pr = one -> representation ();
    char *res = new char [strlen (pr) + strlen ("EXISTS ()") + 1];
    sprintf (res, "EXISTS (%s)", pr);
    delete [] pr;
    return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* PExists::transform(CState* s)
{
  Predicate* first = (Predicate*) One() -> transform(s);
  PExists* temp = new PExists(first,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:	(CState*, CIEnvStackDataType*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.
//
//////////////////////////////////////////////////////////////////////////////

NonType* PExists::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  Predicate* first = (Predicate*) One() -> transformDeep(s, startingPt);
  PExists* temp = new PExists(first,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int PExists::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_EXISTS))
	{
	  // do it for first child.
	  if (one->compares(((PExists*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((PExists*)nt,((PExists*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((PVar*)one)->Name()))->Child_Look_Up())->deepeq(((PExists*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This match method is used for matching statement.
// No duplicated variable entry is allowed.
//
//////////////////////////////////////////////////////////////////////////////

int PExists::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_EXISTS))
	{
	  // do it for first child.
	  if (one->comparesSTM(((PExists*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((PExists*)nt,((PExists*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((PVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
} 

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list and inverse dependency list
//
//////////////////////////////////////////////////////////////////////////////

void PExists::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on inverse dependency list.
// undo for dependency list is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void PExists::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void PExists::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);

}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Implements the existentially quantifying predicate former.
//
//////////////////////////////////////////////////////////////////////////////

koBool *PExists::invoke(koObject* x)
{
  void *handle = dlopen(pexistOpt, RTLD_LAZY);
  void *symbol = dlsym(handle, "callOpt");

  NonType *result = ((callOptPtr)symbol)(new BInvoke(this,x));
  dlclose(handle);
  return (koBool*)result;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Implements the existentially quantifying predicate former.
//
//////////////////////////////////////////////////////////////////////////////

OSBool* PExists::exec(OSObject* x)
{
  Iterator *i = ((Iterator*)x);
  OSBPrimitive *b;
  int stop = 0;

  i->open();
  while(!stop && !(i->finished())) {
    OSObject *r = i->next();
    b = (OSBPrimitive*)(One()->invoke(r->kola()))->result();
    stop = b->Val();
  }

  i->close();
  return new OSBPrimitive(b->Val());
}
#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* PExists::prolog ()
{
  char *pr = one -> prolog ();
  char *res = new char [strlen (pr) + strlen ("pexists()") + 1];
  sprintf (res, "pexists(%s)", pr);
  delete [] pr;
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref PExists::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("pexists"),1,or);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PExists::sql (koObject *o)
{
    OVar *x   = new OVar (0); 

    char *res;
    char *pr = (one -> inv (x)) -> sql ();
	char *in = o -> sql ();

    if (strncmp (pr, "ERROR", 5) == 0) {
	  delete [] in;
	  delete x;
	  return pr;
    } else if (strncmp (in, "ERROR", 5) == 0) {
	  delete [] pr;
	  delete x;
	  return in;
    } else {
	  char *res = new char [strlen ("(exists  in : )") + strlen (x -> Name ()) + strlen (in) + strlen (pr) + 1];
	  sprintf (res, "(exists %s in %s: %s)", x -> Name (), in, pr);
	  delete [] pr;
	  delete [] in;
	  delete x;
	  return res;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PExists::oql (koObject *o)
{
    OVar *x   = new OVar (0); 

    char *res;
    char *pr = (one -> inv (x)) -> oql ();
	char *in = o -> oql ();

    if (strncmp (pr, "ERROR", 5) == 0) {
	  delete [] in;
	  delete x;
	  return pr;
    } else if (strncmp (in, "ERROR", 5) == 0) {
	  delete [] pr;
	  delete x;
	  return in;
    } else {
	  char *res = new char [strlen ("(exists  in : )") + strlen (x -> Name ()) + strlen (in) + strlen (pr) + 1];
	  sprintf (res, "(exists %s in %s: %s)", x -> Name (), in, pr);
	  delete [] pr;
	  delete [] in;
	  delete x;
	  return res;
    }
}
