/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)					 	 
// FILE: KOPUser.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOPUser.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PUser
// Parameters:	(char *n)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PUser::PUser (char *s, TPred * t)  : Predicate (t, PRED_USER, s)
{
  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PUser
// Parameters:	(PUser& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PUser::PUser (const PUser& other) : Predicate ((Predicate &) other)
{
  if (other.val == NULL) val = NULL;
  else {
    val = new char [strlen(other.val) + 1];
    sprintf (val, "%s", other.val);
  }
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~PUser
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PUser::~PUser ()
{
 if (val) {delete [] val; val = NULL;}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Predicate*
//         								    
//////////////////////////////////////////////////////////////////////////////

Predicate* PUser::Copy () const
{
  PUser* temp = new PUser (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////


int PUser::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == PNAME ) && (name_->pn == PRED_USER))
  {
    delete name_;
    if (!strcmp(((PUser*)other)->Name(),namestring))
      return T;
    else
      return F; 
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 5 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* PUser::Myname()
{
	NameUnion* a = new NameUnion(PNAME, PRED_USER);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)						  
//
//  Return
//   new BResBBox (name + " ? " + a -> reduce ())
//         								    
//////////////////////////////////////////////////////////////////////////////

// BResult* PUser::reduce (OResult* a)
// {

//     BResBB *res = new BResBB ((char *) Name(), a -> Copy (), T);
//     delete a;
//     return res;
    
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							 
//
//  "PPRIM: USER"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PUser::representation ()
{
	char *tmp = (type->Dom())->representation(); 
    char *res = new char [strlen (Name()) + + strlen(tmp) + strlen ("Pred(, )") + 1];
    sprintf (res, "Pred(%s, %s)", Name(), tmp);
	if (tmp) { delete [] tmp; tmp = NULL; }
    return res;

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* PUser::transform(CState* s)
{
    
  PUser* temp = new PUser(val,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair wehave.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int PUser::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_USER) && (!strcmp(((PUser*)nt)->Name(),val)))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}
  
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void PUser::replaceChild(NonType* child, int which_child)
{
}  

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSBool*)
//
//  Executes the user-defined method specified by Name().
//
//////////////////////////////////////////////////////////////////////////////

OSBool* PUser::exec(OSObject* x)
{
  predptr code = NULL;
  os_class_type *class_type = ((os_class_type*)os_type::type_at(x));
  const char *t = class_type->get_name();

  while (code == NULL) {
    code = PTableLookup(t, Name());

    // Look for the class's base class (if any).  This is necessary
    // in the event that a class's method is inherited from the
    // base class' definition (virtual function is not redefined).

    if (code == NULL) {
      os_Cursor<os_base_class*>class_list(class_type->get_base_classes());
      os_base_class *b = class_list.first();

      // if there are no base classes, function name is incorrect
      if (b == 0)
        throw NoSuchFuncErr();

      os_class_type &c = b->get_class();
      class_type = &c;
      t = c.get_name();
    }
  }

  return (*code)(x);
}
#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* PUser::prolog ()
{
  char *tmp;
  if (type == NULL) {
	tmp = new char [strlen ("null") + 1];
	sprintf (tmp, "%s", "null");
  } else {	
	tmp = (type->Dom())->prolog();
  };

  char *res = new char [strlen (val) + strlen ("pred(k, )") + strlen(tmp) + 1];
  sprintf (res, "pred(k%s, %s)", val,tmp);
  if (tmp) { delete [] tmp; tmp = NULL; }  
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref PUser::sp_prolog ()
{
  SP_term_ref dom;
  if (type != NULL) {
	dom = (type->Dom())->sp_prolog();
  } else {
	dom = SP_new_term_ref();
	SP_put_string(dom,"knull");
  };

  char *tmp = new char [strlen(Name()) + 2];
  sprintf(tmp,"k%s",Name());

  SP_term_ref nam = SP_new_term_ref();
  SP_term_ref res = SP_new_term_ref();

  SP_put_string(nam,tmp);

  SP_cons_functor(res, SP_atom_from_string("pred"),2,nam,dom);

  if (tmp) {delete tmp; tmp = NULL;}

  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PUser::sql (koObject *o)
{    
  char *arg1, *arg2;

  NameUnion *nu = o -> Myname ();
  if (nu ->on == OBJ_PAIR) {
    delete nu;
	arg1 = o -> asPair () -> One () -> sql ();
	
	koObject *o2 = o -> asPair () -> Two ();
	nu = o2 -> Myname ();
	if (nu ->on == OBJ_PAIR) 
	  arg2 = o2 -> asPair () -> asargs ();
	else
	  arg2 = o2 -> sql ();
	delete nu;

    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
      return arg1;
	} else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
      return arg2;
	} else {
      char *res = new char [strlen (". (%s)") + strlen (val) + strlen (arg1) + strlen (arg2) + 1];
      sprintf (res, "%s.%s (%s)", arg1, val, arg2);
      delete [] arg1;
      delete [] arg2;
	  return res;
    }
  } else {
    delete nu;
	char *arg = o -> sql ();
	char *res = new char [strlen (".") + strlen (val) + strlen (arg) + 1];
	sprintf (res, "%s.%s", arg, val);
    delete [] arg;
	return res;
  }	
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PUser::oql (koObject *o)
{    
  char *arg1, *arg2;

  NameUnion *nu = o -> Myname ();
  if (nu ->on == OBJ_PAIR) {
    delete nu;
	arg1 = o -> asPair () -> One () -> oql ();
	
	koObject *o2 = o -> asPair () -> Two ();
	nu = o2 -> Myname ();
	if (nu ->on == OBJ_PAIR) 
	  arg2 = o2 -> asPair () -> asargs ();
	else
	  arg2 = o2 -> oql ();
	delete nu;

    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
      return arg1;
	} else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
      return arg2;
	} else {
      char *res = new char [strlen (". (%s)") + strlen (val) + strlen (arg1) + strlen (arg2) + 1];
      sprintf (res, "%s.%s (%s)", arg1, val, arg2);
      delete [] arg1;
      delete [] arg2;
	  return res;
    }
  } else {
    delete nu;
	char *arg = o -> oql ();
	char *res = new char [strlen (".") + strlen (val) + strlen (arg) + 1];
	sprintf (res, "%s.%s", arg, val);
    delete [] arg;
	return res;
  }	
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PUser::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}
