/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: TYTSet.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "TYTSet.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TSet
// Parameters:	(TObj* t)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TSet::TSet (TObj* t)
{

    one = t; 
    myname = TYPE_SET;
   
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TSet
// Parameters:	(TSet& other)
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TSet::TSet (TSet& other)
{

    one = (other . One ()) -> Copy ();
    myname = TYPE_SET;

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~TSet
// Parameters:	()
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TSet::~TSet ()
{

  if (one) { delete one; one = NULL; }
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  Return copy of this as Bool
//         								    
//////////////////////////////////////////////////////////////////////////////

TObj* TSet::Copy ()
{

    return new TSet (*this);
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  return "SET [one -> representation ()]"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TSet::representation ()
{

    char *orr = one -> representation ();
    char *res = new char [strlen (orr) + strlen ("{}") + 1];
    sprintf (res, "{%s}", orr);
    delete [] orr;
    return res;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  prolog
// Parameters:	()							  
//
//  return "SET [one -> prolog ()]"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TSet::prolog ()
{
  char *orr;
  if (one == NULL) {
	orr = new char [strlen ("knull") + 1];
	sprintf (orr, "knull");
  } else
    orr = one -> prolog ();
  char *res = new char [strlen (orr) + strlen ("tset()") + 1];
  sprintf (res, "tset(%s)", orr);
  delete [] orr;
  return res;    
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref TSet::sp_prolog ()
{
  SP_term_ref or;
  if (one == NULL) {
	cout << "Calling sp_prolog on TSet with child = NULL: could be a problem" << endl;
	or = NULL;
  } else
	or = one -> sp_prolog ();

  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("tset"),1,or);
  return res;
}
#endif // _PROLOG_




